#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "wndAddrBook.h"

/* **************************************************************************
 * CALLBACK FUNCTIONS
 ************************************************************************** */

void
on_wndAddrBook_show                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *tvContactList = lookup_widget(GTK_WIDGET(widget), "tvContactList");
	GtkWidget *cbContactServer = lookup_widget(GTK_WIDGET(widget), "cbContactServer");
	
	DEBUG( printf("on_wndAddrBook_show()\n"); );
	nt_create_tvContactList(tvContactList);
	nt_fill_cbServer(cbContactServer);
	AppCfg.flags |= FLAG_SHOWADDRBOOK;
}


void
on_wndAddrBook_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	AppCfg.flags &= ~FLAG_SHOWADDRBOOK;
}


void
on_clBtChange_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *treeview = lookup_widget(GTK_WIDGET(button), "tvContactList");
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_selected_foreach(selection, nt_get_contact_data, treeview);
}


void
on_clBtCancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *wnd = lookup_widget(GTK_WIDGET(button), "wndAddrBook");
	gtk_widget_destroy(GTK_WINDOW(wnd));
}



void
on_clBtOk_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{

}

/* **************************************************************************
 * HELPER FUNCTIONS
 ************************************************************************** */


int nt_fill_tvContactList(GtkWidget *widget)
{
  FILE *fp = fopen(AppCfg.file_addrbook, "rt");
  char buf[MAX_CFGLINE_LEN], *name, *prefix, *phone, *server;
  GtkTreeIter  iter;
  
  if ( !fp )
  	return -1;
  
  DEBUG( printf("nt_fill_tvContactList()\n"); );

  while (fgets(buf, MAX_CFGLINE_LEN, fp)) {
	buf[strlen(buf)-1] = '\0';
  	name = buf;
  	if ( *name != '#' ) {
  		prefix = strchr(name, ':');
  		if ( prefix ) {
  			*prefix = '\0';
			prefix++;
			phone = strchr(prefix, ':');
			if ( phone ) {
				*phone = '\0';
				phone++;
				server = strchr(phone, ':');
				if ( server ) {
					*server = '\0';
					*server++;
					gtk_list_store_append (widget, &iter);
					gtk_list_store_set (widget, &iter,
						  COLUMN_CONTACTNAME, name,
						  COLUMN_CONTACTPREFIX, prefix,
						  COLUMN_CONTACTPHONE, phone,
						  COLUMN_CONTACTSERVER, server,
						  -1);
				}
			}
  		}
  	}
  }
  fclose(fp);
  return 0;
}




void nt_get_contact_data      (GtkTreeModel *model,  
                                       GtkTreePath *path,
                                       GtkTreeIter *iter,
                                       gpointer widget)
{
	gchar *name, *phone, *prefix, *server;
	GtkWidget *ceContactName, *ceContactPrefix, *ceContactPhone, *ceContactServer;


	DEBUG( printf("CLICKED\n"); );
	gtk_tree_model_get(model, iter, COLUMN_CONTACTNAME, &name, COLUMN_CONTACTPREFIX, &prefix, COLUMN_CONTACTPHONE, &phone, COLUMN_CONTACTSERVER, &server, -1);
	if ( name && phone && server && prefix ) {
		ceContactName = lookup_widget(GTK_WIDGET(widget), "ceContactName");
		ceContactPrefix = lookup_widget(GTK_WIDGET(widget), "ceContactPrefix");
		ceContactPhone = lookup_widget(GTK_WIDGET(widget), "ceContactPhone");
		ceContactServer = lookup_widget(GTK_WIDGET(widget), "ceContactServer");
		gtk_entry_set_text(GTK_ENTRY(ceContactName), name);
		gtk_entry_set_text(GTK_ENTRY(ceContactPrefix), prefix);
		gtk_entry_set_text(GTK_ENTRY(ceContactPhone), phone);
		gtk_entry_set_text(GTK_ENTRY(ceContactServer), server);
	}
	if ( name ) g_free(name);
	if ( phone ) g_free(phone);
	if ( server ) g_free(server);
	if ( prefix ) g_free(prefix);
}


int nt_create_tvContactList(GtkWidget *widget)
{
	GtkWidget *tvContactList = lookup_widget(GTK_WIDGET(widget), "tvContactList");
	GtkListStore *store;
	GtkTreeModel *model;

	/* create list store */
	store = gtk_list_store_new (CONTACT_NUM_COLUMNS,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING);

	nt_fill_tvContactList(store);

	model = GTK_TREE_MODEL (store);

	gtk_tree_view_set_model(GTK_TREE_VIEW(tvContactList), model);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tvContactList), TRUE);
//	gtk_tree_view_set_search_column (GTK_TREE_VIEW (tvContactList),
//				       COLUMN_CONTACTNAME);

	g_object_unref (model);

	/* add columns to the tree view */
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	/* column for Contact Name */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes ("Name",
						     renderer,
						     "text", COLUMN_CONTACTNAME,
						     NULL);

	/* set this column to a fixed sizing (of 50 pixels) */
	//gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
	//			   GTK_TREE_VIEW_COLUMN_FIXED);
	//gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 70);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_CONTACTNAME);
	gtk_tree_view_append_column (widget, column);

	/* column for Contact Prefix */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Prefix",
						     renderer,
						     "text",
						     COLUMN_CONTACTPREFIX,
						     NULL);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_CONTACTPREFIX);
	gtk_tree_view_append_column (widget, column);

	/* column for Contact Phone # */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Phone #",
						     renderer,
						     "text",
						     COLUMN_CONTACTPHONE,
						     NULL);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_CONTACTPHONE);
	gtk_tree_view_append_column (widget, column);

	/* column for Contact's Preferred Server */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Server",
						     renderer,
						     "text",
						     COLUMN_CONTACTSERVER,
						     NULL);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_CONTACTSERVER);
	gtk_tree_view_append_column (widget, column);

	return 0;
}

