#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "wndPrefs.h"

/* **************************************************************************
 * CALLBACK FUNCTIONS
 ************************************************************************** */

void
on_wndPrefs_show                       (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *wnd = lookup_widget(GTK_WIDGET(widget), "wndPrefs");
	GtkWidget *ceConfs = lookup_widget(GTK_WIDGET(widget), "ceConfs");
	GtkWidget *ceAddrBook = lookup_widget(GTK_WIDGET(widget), "ceAddrBook");
	GtkWidget *ceServers = lookup_widget(GTK_WIDGET(widget), "ceServers");
	GtkWidget *ceBinary = lookup_widget(GTK_WIDGET(widget), "ceBinary");
	GtkWidget *ceMyNumber = lookup_widget(GTK_WIDGET(widget), "ceMyNumber");
	GtkWidget *ceMyAlias = lookup_widget(GTK_WIDGET(widget), "ceMyAlias");
	GtkWidget *ceProxy = lookup_widget(GTK_WIDGET(widget), "ceProxy");
	GtkWidget *bUseSignature = lookup_widget(GTK_WIDGET(widget), "bUseSignature");
	GtkWidget *ceSignature = lookup_widget(GTK_WIDGET(widget), "ceSignature");
	GtkWidget *cbDefServer = lookup_widget(GTK_WIDGET(widget), "cbDefServer");
	
	gtk_entry_set_text(GTK_ENTRY(ceConfs), AppCfg.file_conf);
	gtk_entry_set_text(GTK_ENTRY(ceAddrBook), AppCfg.file_addrbook);
	gtk_entry_set_text(GTK_ENTRY(ceServers), AppCfg.file_passwd);
	gtk_entry_set_text(GTK_ENTRY(ceBinary), AppCfg.file_bin);
	
	nt_load_linuxsmsrc();
	
	gtk_entry_set_text(GTK_ENTRY(ceMyNumber), AppCfg.def_mynumber);
	gtk_entry_set_text(GTK_ENTRY(ceMyAlias), AppCfg.def_myalias);
	gtk_entry_set_text(GTK_ENTRY(ceProxy), AppCfg.def_proxy);
	gtk_entry_set_text(GTK_ENTRY(ceSignature), AppCfg.def_signature);
	nt_fill_cbServer(cbDefServer);
	
	AppCfg.flags |= FLAG_SHOWPREFS;
}


void
on_wndPrefs_destroy                    (GtkObject       *object,
                                        gpointer         user_data)
{
	AppCfg.flags &= ~FLAG_SHOWPREFS;
}


void
on_btOk_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *wnd = lookup_widget(GTK_WIDGET(button), "wndPrefs");
	GtkWidget *ceConfs = lookup_widget(GTK_WIDGET(button), "ceConfs");
	GtkWidget *ceAddrBook = lookup_widget(GTK_WIDGET(button), "ceAddrBook");
	GtkWidget *ceServers = lookup_widget(GTK_WIDGET(button), "ceServers");
	GtkWidget *ceBinary = lookup_widget(GTK_WIDGET(button), "ceBinary");
	GtkWidget *ceMyNumber = lookup_widget(GTK_WIDGET(button), "ceMyNumber");
	GtkWidget *ceMyAlias = lookup_widget(GTK_WIDGET(button), "ceMyAlias");
	GtkWidget *ceProxy = lookup_widget(GTK_WIDGET(button), "ceProxy");
	GtkWidget *bUseSignature = lookup_widget(GTK_WIDGET(button), "bUseSignature");
	GtkWidget *ceSignature = lookup_widget(GTK_WIDGET(button), "ceSignature");
//	GtkWidget *cbDefServer = lookup_widget(GTK_WIDGET(button), "cbDefServer");
	GtkWidget *ceDefServer = lookup_widget(GTK_WIDGET(button), "ceDefServer");
	
	strncpy(AppCfg.file_conf, gtk_entry_get_text(GTK_ENTRY(ceConfs)), MAX_CFGFILEPATH);
	strncpy(AppCfg.file_addrbook, gtk_entry_get_text(GTK_ENTRY(ceAddrBook)), MAX_CFGFILEPATH);
	strncpy(AppCfg.file_passwd, gtk_entry_get_text(GTK_ENTRY(ceServers)), MAX_CFGFILEPATH);
	strncpy(AppCfg.file_bin, gtk_entry_get_text(GTK_ENTRY(ceBinary)), MAX_CFGFILEPATH);
	
	strncpy(AppCfg.def_mynumber, (gchar *)gtk_entry_get_text(GTK_ENTRY(ceMyNumber)), MAX_CFGCONTACT_LEN);
	strncpy(AppCfg.def_myalias, (gchar *)gtk_entry_get_text(GTK_ENTRY(ceMyAlias)), MAX_CFGCONTACT_LEN);
	strncpy(AppCfg.def_server, (gchar *)gtk_entry_get_text(GTK_ENTRY(ceDefServer)), MAX_CFGSERVER_LEN);
	strncpy(AppCfg.def_proxy, (gchar *)gtk_entry_get_text(GTK_ENTRY(ceProxy)), MAX_CFGPROXY_LEN);
	strncpy(AppCfg.def_signature, (gchar *)gtk_entry_get_text(GTK_ENTRY(ceSignature)), MAX_CFGFILEPATH);
	
	nt_save_linuxsmsrc();
	
	gtk_widget_destroy(GTK_WINDOW(wnd));
}



void
on_btCancel_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *wnd = lookup_widget(GTK_WIDGET(button), "wndPrefs");
	gtk_widget_destroy(GTK_WINDOW(wnd));
}


/* **************************************************************************
 * HELPER FUNCTIONS
 ************************************************************************** */



int nt_load_linuxsmsrc()
{
  FILE *fp = fopen(AppCfg.file_conf, "rt");
  char buf[MAX_CFGLINE_LEN], *p, *q;
  GList *items = NULL;
  int i = 0;
  
  if ( !fp )
  	return 1;
  
  while (fgets(buf, MAX_CFGLINE_LEN, fp)) {
  	p = buf;
  	while ((*p) && ((*p==' ') || (*p=='\n')))
  		p++;
  	if ( *p != '#' ) {
  		q = strchr(p, '=');
  		if ( q ) {
	  		buf[strlen(buf)-1] = '\0';
  			*q = '\0'; q++;
  			while ( (*q) && (*q==' '))
  				q++;
  			if ( !strcmp(p, "MYNUMBER") )
		  		strcpy(AppCfg.def_mynumber, q);
		  	else if ( !strcmp(p, "MYALIAS") )
		  		strcpy(AppCfg.def_myalias, q);
		  	else if ( !strcmp(p, "SERVER") )
		  		strcpy(AppCfg.def_server, q);
		  	else if ( !strcmp(p, "PROXY") )
		  		strcpy(AppCfg.def_proxy, q);
		  	else if ( !strcmp(p, "SIGNATURE") )
		  		strcpy(AppCfg.def_signature, q);
  		}
  	}
  }
  fclose(fp);
  return 0;
}

/* TODO: having an existing .rc file is not strictly necessarily, so 
 * don't return if it opens. Just ignore it
 */
int nt_save_linuxsmsrc()
{
  FILE *fold = fopen(AppCfg.file_conf, "rt"), *fnew;
  char buf[MAX_CFGFILEPATH], buf2[MAX_CFGLINE_LEN], *p, *q;

  sprintf(buf,"%s.new", AppCfg.file_conf);
  fnew = fopen(buf, "wt");

  if ( (!fold) || (!fnew) )
  	return 1;
  
  while (fgets(buf2, MAX_CFGLINE_LEN, fold)) {
  	p = buf2;
  	while ((*p) && ((*p==' ') || (*p=='\n')))
  		p++;
  	if ( *p == '#' ) 
  		fprintf(fnew,buf2);
  	else {	
  		q = strchr(buf2, '=');
  		if ( q ) {
  			strncpy(buf, buf2, MAX_CFGFILEPATH);
  			*q = '\0'; q++;
  			while ( (*q) && (*q==' '))
  				q++;
  			if ( !strcmp(p, "MYNUMBER") )
		  		fprintf(fnew, "MYNUMBER=%s\n", AppCfg.def_mynumber);
		  	else if ( !strcmp(p, "MYALIAS") )
		  		fprintf(fnew, "MYALIAS=%s\n", AppCfg.def_myalias);
		  	else if ( !strcmp(p, "SERVER") )
		  		fprintf(fnew, "SERVER=%s\n", AppCfg.def_server);
		  	else if ( !strcmp(p, "PROXY") )
		  		fprintf(fnew, "PROXY=%s\n", AppCfg.def_proxy);
		  	else if ( !strcmp(p, "SIGNATURE") )
		  		fprintf(fnew, "SIGNATURE=%s\n", AppCfg.def_signature);
		  	else fprintf(fnew, buf);
  		}
  	}
  }
  fclose(fold);
  fclose(fnew);
  return 0;
}

