#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "wndServers.h"

/* **************************************************************************
 * CALLBACK FUNCTIONS
 ************************************************************************** */

void
on_wndServers_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *tvServerList = lookup_widget(GTK_WIDGET(widget), "tvServerList");
	
	DEBUG( printf("on_wndServers_show()\n"); );
	nt_create_tvServerList(tvServerList);
	AppCfg.flags |= FLAG_SHOWSERVERS;
}



void
on_wndServers_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
	AppCfg.flags &= ~FLAG_SHOWSERVERS;
}


void
on_tvServerList_cursor_changed         (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	DEBUG( printf("CURSOR CHANGED"); );
}


void
on_tvServerList_row_activated          (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	DEBUG( printf("ROW ACTIVATED"); );
}


gboolean
on_tvServerList_toggle_cursor_row      (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
  DEBUG( printf("callback: on_tvServerList_toggle_cursor_row\n"); );

  return FALSE;
}


void
on_slBtChange_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *treeview = lookup_widget(GTK_WIDGET(button), "tvServerList");
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_selected_foreach(selection, nt_get_server_data, treeview);
}


void
on_slBtCancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *wnd = lookup_widget(GTK_WIDGET(button), "wndServers");
	gtk_widget_destroy(GTK_WINDOW(wnd));
}


void
on_slbtOk_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{

}


/* **************************************************************************
 * HELPER FUNCTIONS
 ************************************************************************** */

int nt_create_tvServerList(GtkWidget *widget)
{
	GtkWidget *tvServerList = lookup_widget(GTK_WIDGET(widget), "tvServerList");
	GtkListStore *store;
	GtkTreeModel *model;

	/* create list store */
	store = gtk_list_store_new (CONTACT_NUM_COLUMNS,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING);

	nt_fill_tvServerList(store);

	model = GTK_TREE_MODEL (store);

	gtk_tree_view_set_model(GTK_TREE_VIEW(tvServerList), model);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tvServerList), TRUE);
//	gtk_tree_view_set_search_column (GTK_TREE_VIEW (tvServerList),
//				       COLUMN_SERVERNAME);

	g_object_unref (model);

	/* add columns to the tree view */
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	/* column for Server Name */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes ("Name",
						     renderer,
						     "text", COLUMN_SERVERNAME,
						     NULL);

	/* set this column to a fixed sizing (of 50 pixels) */
	//gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
	//			   GTK_TREE_VIEW_COLUMN_FIXED);
	//gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 70);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_SERVERNAME);
	gtk_tree_view_append_column (widget, column);

	/* column for Server Login */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Login",
						     renderer,
						     "text",
						     COLUMN_SERVERLOGIN,
						     NULL);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_SERVERLOGIN);
	gtk_tree_view_append_column (widget, column);

	/* column for Server Password */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Password",
						     renderer,
						     "text",
						     COLUMN_SERVERPASSWD,
						     NULL);
	gtk_tree_view_column_set_sort_column_id (column, COLUMN_SERVERPASSWD);
	gtk_tree_view_append_column (widget, column);

	return 0;
}




int nt_fill_tvServerList(GtkWidget *widget)
{
  FILE *fp = fopen(AppCfg.file_passwd, "rt");
  char buf[MAX_CFGLINE_LEN], *name, *user, *passwd;
  GtkTreeIter  iter;
  
  if ( !fp )
  	return -1;
  
  while (fgets(buf, MAX_CFGLINE_LEN, fp)) {
	buf[strlen(buf)-1] = '\0';
  	name = buf;
  	if ( *name != '#' ) {
  		user = strchr(name, ':');
  		if ( user ) {
  			*user = '\0';
			user++;
			passwd = strchr(user, ':');
			if ( passwd ) {
				*passwd = '\0';
				passwd++;
				gtk_list_store_append (widget, &iter);
				gtk_list_store_set (widget, &iter,
					  COLUMN_SERVERNAME, name,
					  COLUMN_SERVERLOGIN, user,
					  COLUMN_SERVERPASSWD, passwd,
					  -1);
			}
  		}
  	}
  }
  fclose(fp);
  return 0;
}


int nt_save_tvServerList(GtkWidget *widget)
{
  FILE *fp; ;
  char buf[MAX_CFGLINE_LEN];
  GtkTreeIter  iter;
  
  sprintf(buf, "%s.new", AppCfg.file_passwd);
  if ( !(fp==fopen(buf, "rt")) )
  	return -1;
  
  while (1) {
  	break;
  }
  fclose(fp);
  return 0;
}


void nt_get_server_data      (GtkTreeModel *model,  
                                       GtkTreePath *path,
                                       GtkTreeIter *iter,
                                       gpointer widget)
{
	gchar *name, *login, *passwd;
	GtkWidget *ceServerName, *ceServerLogin, *ceServerPasswd;


	DEBUG( printf("CLICKED\n"); );
	gtk_tree_model_get(model, iter, COLUMN_SERVERNAME, &name, COLUMN_SERVERLOGIN, &login, COLUMN_SERVERPASSWD, &passwd, -1);
	if ( name && login && passwd ) {
		ceServerName = lookup_widget(GTK_WIDGET(widget), "ceServerName");
		ceServerLogin = lookup_widget(GTK_WIDGET(widget), "ceServerLogin");
		ceServerPasswd = lookup_widget(GTK_WIDGET(widget), "ceServerPasswd");
		gtk_entry_set_text(GTK_ENTRY(ceServerName), name);
		gtk_entry_set_text(GTK_ENTRY(ceServerLogin), login);
		gtk_entry_set_text(GTK_ENTRY(ceServerPasswd), passwd);
	}
	if ( name ) g_free(name);
	if ( login ) g_free(login);
	if ( passwd ) g_free(passwd);
}



